#include "as31glue.h"
#include "printf.h"
#include "paulmon2.h"
#include "display.h"
#include "malloc.h"
#include "playlist.h"
#include "string.h"


simm_id first_file;	/* of the active playlist */
simm_id last_file;	/* of the active playlist */

simm_id first_playlist;
simm_id active_playlist;
simm_id last_playlist;

/* limit the maximum subdirectory depth (number of recursive */
/* calls) so that we don't overflow the 8051's small stack */
/* avoiding stack-hogging printf calls will allow deeper subdirs */
#define MAX_SUBDIR_DEPTH 5

//defined in playlist.h

static xdata struct dirent_struct pl_fileinfo;
static unsigned char subdir_depth;


void fill_playlist_setup(void)
{
	subdir_depth=0;
        first_file=0;
        last_file=0;
}


void fill_playlist_by_dir(unsigned long pl_cluster) reentrant
{
	char dir,r;
        simm_id temp_id;
        xdata struct filelist_struct * pl_struct;
        xdata struct filelist_struct * temp_pl_struct;
        
	// Read in and cache this directory

	if (++subdir_depth > MAX_SUBDIR_DEPTH) return;
	dir = file_open_by_1st_cluster(pl_cluster);
	if (dir < 0) {
		subdir_depth--;
		return;		// unable to open directory
	}
	do_ide_loop(1);
	file_cache(dir, 0, 0xFFFFFFFF);  // request entire directory cached
	do {
		do_ide_loop(4);
		r = file_cache_work(dir);
		if (r == 2) pm2_exit();
	} while (r == 0);
	do_ide_loop(8);
	file_seek(dir, 0);

	while(!dir_read(dir, &pl_fileinfo)) {
		if (pl_fileinfo.attrib & 0x10) {
			if(pl_fileinfo.short_name[0] != '.') {
				printf("Enter Dir: %s (depth=%d)\r\n",
				   pl_fileinfo.long_name, (int)subdir_depth);
				lcd_startup_message(pl_fileinfo.long_name);
				fill_playlist_by_dir(pl_fileinfo.cluster);
				printf("Leaving Dir\r\n");
			}
		} else {
                  if(is_mp3_file(&pl_fileinfo))
                    {
  		      if(first_file == 0)
		        {
                          first_file = simm_malloc((unsigned int)(sizeof(struct filelist_struct)));
                          last_file = first_file;
                      
                          pl_struct = addr6(last_file);
                          pl_struct->prev = 0;
                        }
                      else
                        {
                          temp_id = simm_malloc((unsigned int)(sizeof(struct filelist_struct)));
                          // printf("Temp ID Returned : %ld\r\n", temp_id);

                          pl_struct = addr6(temp_id);
                          temp_pl_struct = addr7(last_file);
                      
                          temp_pl_struct->next = temp_id;
                          pl_struct->prev = last_file;
                          last_file = temp_id;
                        }
                      // This code is the same for both empty and not empty list (empty: first_file == 0)
                      pl_struct->cluster = pl_fileinfo.cluster;
                      pl_struct->size = pl_fileinfo.size;
                      strcpy(pl_struct->long_filename, pl_fileinfo.long_name);
                      strcpy(pl_struct->short_filename, pl_fileinfo.short_name);
                      pl_struct->next = 0;

                      /*
                      printf("******* Processing PL Record *******\r\n");
                      printf("Filename: %s\r\n",pl_struct->short_filename);
                      printf("Filename: %s\r\n",pl_struct->long_filename);
                      printf("Filesize: %ld\r\n",pl_struct->size);
                      printf("Cluster: %ld\r\n",pl_struct->cluster);
                      */
		    }
		 }
	}
	file_close(dir);
        // This doesn't seem to work... Maybe we need to give the "actual" length ?!?
        //file_uncache(dir, 0, 0xFFFFFFFF);
	subdir_depth--;
}


void show_playlist(void)
{
  simm_id temp_id;
  xdata struct filelist_struct * pl_struct;

  temp_id = first_file;

  printf("Playlist begin...\r\n");

  while(temp_id != 0)
    {
      pl_struct = addr6(temp_id);
      printf("******* PL Record *******\r\n");
      printf("Filename: %s\r\n",pl_struct->short_filename);
      printf("Filename: %s\r\n",pl_struct->long_filename);
      printf("Filesize: %ld\r\n",pl_struct->size);
      printf("Cluster: %ld\r\n",pl_struct->cluster);
      temp_id = pl_struct->next;
    }
  printf("Playlist end...\r\n");
}


char is_mp3_file(xdata struct dirent_struct * fileinfo)
{
  // We should change this checking to the playlist add by dir function or something :-)
  if ((fileinfo->attrib & 0x1E) != 0) return 0;
  if (fileinfo->short_name[9] != 'M') return 0;
  if (fileinfo->short_name[10] != 'P') return 0;
  if (fileinfo->short_name[11] != '3') return 0;
  return 1;
}


